#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rssmb.h"
#include "RSSMB_AM_Example.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
        error = (fCal);\
        if (error < VI_SUCCESS) {cleanup (error);return 0;}\
    
/* Globals */
static int panelHandle;

ViBoolean grfState;
ViReal64  grfLevel, grfFrequency,
          glfGenFreq, gamDepth;
ViInt32   gshape, gamState;
ViStatus  error;
ViSession io = NULL;
ViChar    error_message[256];
ViChar    error_buffer[256];

/* Local Functions */
int initialize (void);
void cleanup (ViStatus error);
void ShapeChange (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSSMB_AM_Example.uir", PANEL)) < 0)
        return -1;
    
    /* Default Values */
    grfState           = VI_FALSE;
    grfLevel           = -30.0;
    grfFrequency       = 1E9;
    glfGenFreq         = 1e3;
    gshape             = 0;
    gamState           = 0;
    gamDepth           = 30.0;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar idn[256];
    int    smc = 0;
    switch (event)
    {
        case EVENT_COMMIT:
            /* Initialize driver */
            if (io == VI_NULL)
            {
                if (initialize() < VI_SUCCESS) break;
            }

            CHECKERR (rssmb_IdentificationQuery (io, idn));
            smc = (strstr (idn, "SMC") != NULL);
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
            ProcessDrawEvents ();

            SetWaitCursor(1);

            /* Get values from controls */
            GetCtrlVal (panelHandle, PANEL_RF_FREQ, &grfFrequency);
            GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &grfLevel);
            GetCtrlVal (panelHandle, PANEL_RF_STATE, &grfState);
            GetCtrlVal (panelHandle, PANEL_AM_STATE, &gamState);
            GetCtrlVal (panelHandle, PANEL_AM_DEPTH, &gamDepth);            
            GetCtrlVal (panelHandle, PANEL_LFGEN_FREQ, &glfGenFreq);            
            GetCtrlVal (panelHandle, PANEL_LFGEN_SHAPE, &gshape);           
            
            /* Configure RF output part */
            CHECKERR (rssmb_confRFFreq(io, grfFrequency, 0.0, VI_FALSE, 0.0));
            CHECKERR (rssmb_confRFLevel (io, grfLevel, 0.0, 10.0, VI_FALSE, 0.0,
                                        RSSMB_OUTPUT_AMODE_AUTO,
                                        RSSMB_POWER_LEVEL_RCL_EXCLUDE));

            /* Configure common AM settings */
            CHECKERR (rssmb_configAM (io, gamState, RSSMB_MODULATION_SOURCE_INT, gamDepth,
                                      RSSMB_AM_EXTERNAL_COUPLING_AC));
            CHECKERR (rssmb_confLFFreq (io, VI_TRUE, glfGenFreq, 1.0));
            if (!smc)
            {
                CHECKERR (rssmb_SetLFGenShape (io, gshape));
            }
            CHECKERR (rssmb_SetLFOutputState (io, gamState));
            

            /* Turn On/Off RF output at the end of sequence */
            CHECKERR (rssmb_SetOutputState(io, grfState));

            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);

            SetWaitCursor(0);
            break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMB Signal Generator\n"
                              "instrument driver provided in the package.\n");
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            if (io) rssmb_close (io);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;


    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssmb_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssmb_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);

    SetWaitCursor(0);
    return 0;
}

void cleanup (ViStatus error)
{
    char error_buffer[1024];
    char error_message [256];
    
    rssmb_error_message (io, error, error_message);
    sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);
    MessagePopup ("Error", error_buffer); 
    rssmb_error_query (io, &error, error_message);
    sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);
    SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
    SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
    SetWaitCursor (0);
    MessagePopup ("Error", error_buffer);
    rssmb_close(io);
    io = NULL;
    return;
}
